@@ -10,7 +10,7 @@ import warnings
 
 from collections import defaultdict
 
-from pyfits.card import Card, CardList, BLANK_CARD, _pad
+from pyfits.card import Card, CardList, BLANK_CARD, KEYWORD_LENGTH, _pad
 from pyfits.file import _File, PYTHON_MODES
 from pyfits.util import (BLOCK_SIZE, deprecated, isiterable, encode_ascii,
                          decode_ascii, fileobj_mode, fileobj_is_binary,
@@ -123,7 +123,17 @@ class Header(object):
             key = key.upper()
             # Special case for commentary cards
             return _HeaderCommentaryCards(self, key)
-        return self._cards[self._cardindex(key)].value
+        if isinstance(key, tuple):
+            keyword = key[0]
+        else:
+            keyword = key
+        card = self._cards[self._cardindex(key)]
+        if (card.field_specifier is not None and
+            keyword == card.keyword.split('.', 1)[0]):
+            # This is RVKC; if only the top-level keyword was specified return
+            # the raw value, not the parsed out float value
+            return card.rawvalue
+        return card.value
 
     def __setitem__(self, key, value):
         if isinstance(key, slice) or self._haswildcard(key):
@@ -516,7 +526,16 @@ class Header(object):
             A string representing a FITS header.
         """
 
-        s = sep.join(str(card) for card in self._cards)
+        lines = []
+        for card in self._cards:
+            s = str(card)
+            # Cards with CONTINUE cards may be longer than 80 chars; so break
+            # them into multiple lines
+            while s:
+                lines.append(s[:Card.length])
+                s = s[Card.length:]
+
+        s = sep.join(lines)
         if endcard:
             s += sep + _pad('END')
         if padding:
@@ -606,21 +625,21 @@ class Header(object):
                 fileobj.close()
 
     @classmethod
-    def fromtextfile(cls, fileobj):
+    def fromtextfile(cls, fileobj, endcard=False):
         """
         Equivalent to ``Header.fromfile(fileobj, sep='\\n', endcard=False,
         padding=False)``.
         """
 
-        return cls.fromfile(fileobj, sep='\n', endcard=False, padding=False)
+        return cls.fromfile(fileobj, sep='\n', endcard=endcard, padding=False)
 
-    def totextfile(self, fileobj, clobber=False):
+    def totextfile(self, fileobj, endcard=False, clobber=False):
         """
         Equivalent to ``Header.tofile(fileobj, sep='\\n', endcard=False,
         padding=False, clobber=clobber)``.
         """
 
-        self.tofile(fileobj, sep='\n', endcard=False, padding=False,
+        self.tofile(fileobj, sep='\n', endcard=endcard, padding=False,
                     clobber=clobber)
 
     def clear(self):
@@ -766,7 +785,8 @@ class Header(object):
         # Don't try to make a temporary card though if they keyword looks like
         # it might be a HIERARCH card or is otherwise invalid--this step is
         # only for validating RVKCs.
-        if (len(keyword) <= 8 and Card._keywd_FSC_RE.match(keyword) and
+        if (len(keyword) <= KEYWORD_LENGTH and
+            Card._keywd_FSC_RE.match(keyword) and
             keyword not in self._keyword_indices):
             new_card = Card(keyword, value, comment)
             new_keyword = new_card.keyword
@@ -1560,7 +1580,7 @@ class Header(object):
             # Returns the index into _cards for the n-th card with the given
             # keyword (where n is 0-based)
             if keyword and keyword not in self._keyword_indices:
-                if len(keyword) > 8:
+                if len(keyword) > KEYWORD_LENGTH or '.' in keyword:
                     raise KeyError("Keyword %r not found." % keyword)
                 # Great--now we have to check if there's a RVKC that starts
                 # with the given keyword, making failed lookups fairly
@@ -1712,8 +1732,9 @@ class Header(object):
         """
 
         # The maximum value in each card can be the maximum card length minus
-        # the key length minus a space (or just minus one if the key is blank)
-        maxlen = Card.length - len(keyword) - 1
+        # the maximum key length (which can include spaces if they key length
+        # less than 8
+        maxlen = Card.length - KEYWORD_LENGTH
         valuestr = str(value)
 
         if len(valuestr) <= maxlen:
@@ -1770,7 +1791,8 @@ class Header(object):
                     pass
 
         for name in ('SIMPLE', 'XTENSION', 'BITPIX', 'NAXIS', 'EXTEND',
-                     'PCOUNT', 'GCOUNT', 'GROUPS', 'BSCALE', 'TFIELDS'):
+                     'PCOUNT', 'GCOUNT', 'GROUPS', 'BSCALE', 'BZERO',
+                     'TFIELDS'):
             try:
                 del self[name]
             except KeyError:
@@ -2008,10 +2030,10 @@ class _HeaderComments(_CardAccessor):
     def __repr__(self):
         """Returns a simple list of all keywords and their comments."""
 
-        keyword_width = 8
+        keyword_length = KEYWORD_LENGTH
         for card in self._header._cards:
-            keyword_width = max(keyword_width, len(card.keyword))
-        return '\n'.join('%*s  %s' % (keyword_width, c.keyword, c.comment)
+            keyword_length = max(keyword_length, len(card.keyword))
+        return '\n'.join('%*s  %s' % (keyword_length, c.keyword, c.comment)
                          for c in self._header._cards)
 
     def __getitem__(self, item):
