@@ -12,7 +12,7 @@ import pyfits
 from pyfits.convenience import _getext
 from pyfits.util import BytesIO
 from pyfits.tests import PyfitsTestCase
-from pyfits.tests.util import catch_warnings, ignore_warnings
+from pyfits.tests.util import catch_warnings, ignore_warnings, CaptureStdio
 
 from nose.tools import assert_equal, assert_raises, assert_true, assert_false
 
@@ -547,6 +547,20 @@ class TestStreamingFunctions(PyfitsTestCase):
             shdu = self._make_streaming_hdu(f)
             shdu.write(arr)
 
+    def test_fix_invalid_extname(self):
+        phdu = pyfits.PrimaryHDU()
+        ihdu = pyfits.ImageHDU()
+        ihdu.header['EXTNAME'] = 12345678
+        hdul = pyfits.HDUList([phdu, ihdu])
+
+        assert_raises(pyfits.VerifyError, hdul.writeto, self.temp('temp.fits'),
+                      output_verify='exception')
+        with CaptureStdio():
+            hdul.writeto(self.temp('temp.fits'), output_verify='fix')
+        with pyfits.open(self.temp('temp.fits')):
+            assert_equal(hdul[1].name, '12345678')
+            assert_equal(hdul[1].header['EXTNAME'], '12345678')
+
     def _make_streaming_hdu(self, fileobj):
         hd = pyfits.Header()
         hd['SIMPLE'] = (True, 'conforms to FITS standard')
